/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package unikl.disco.nc.gui;

import java.util.logging.Logger;
import org.jfree.data.DomainInfo;
import org.jfree.data.DomainOrder;
import org.jfree.data.Range;
import org.jfree.data.RangeInfo;
import org.jfree.data.xy.XYSeriesCollection;
import unikl.disco.math.Util;
import unikl.disco.nc.Curve;
import unikl.disco.nc.LinearSegment;

/**
 *
 * @author ngollan
 */
public class CurveDataset extends XYSeriesCollection implements DomainInfo, RangeInfo {
	private static Logger log;

	private static double domainUpperBound = Double.NaN;

	public CurveDataset() {
		super();

		log = Logger.getLogger("CurveDataset");
	}

	@Override
	public Range getDomainBounds(boolean includeInterval) {
		return new Range(getDomainLowerBound(includeInterval), getDomainUpperBound(includeInterval));
	}

	@Override
	public double getDomainLowerBound(boolean includeInterval) {
		return 0.0;
	}

	public double getRangeLowerBound(boolean includeInterval) {
		return 0.0;
	}

	public double getRangeUpperBound(boolean includeInterval) {
		log.fine("Calculating range upper bound...");
		if(!Util.isFinite(domainUpperBound)) {
			getDomainUpperBound(false);
		}

		double ret = 0.0;
		for(int i=0; i<this.getSeriesCount(); ++i) {
			if(getSeries(i) instanceof Curve) {
				Curve c = (Curve)getSeries(i);

				double bound = c.getMaxY(0, domainUpperBound);
				if(Util.isFinite(bound)) {
					ret = Math.max(ret, bound);
					log.fine("Range bound: " + ret + " ...");
				} else {
					log.fine("Infinite bound: " + bound + " ...");
				}
			}
		}

		log.fine("Returning range bound " + ret);
		return ret;
	}

	public Range getRangeBounds(boolean includeInterval) {
		return new Range(getRangeLowerBound(includeInterval), getRangeUpperBound(includeInterval));
	}

	@Override
	public double getDomainUpperBound(boolean includeInterval) {
		log.fine("Calculating domain upper bound...");
		//TODO: it may be useful to check if the set contains bounds and then
		// limit the domain to show the bounds plus some surroundings.

		// For now, calculate the domain bounds so that at least one full
		// period of each curve in the dataset is shown.
		double ret = 0.0;

		for(int i=0; i<this.getSeriesCount(); ++i) {
			if(getSeries(i) instanceof Curve) {
				Curve c = (Curve)getSeries(i);

				ret = Math.max(ret, 1.1*c.getSegment(-1).x0);
			}

			log.fine("Domain bound: " + ret + " ...");
		}

		domainUpperBound = ret;
		log.fine("Returning domain bound " + ret);
		return ret;
	}

	@Override
	public DomainOrder getDomainOrder() {
		return DomainOrder.ASCENDING;
	}

	public double getYPointValue(int series, int item) {
		if(getSeries(series) instanceof Curve) {
			Curve c = (Curve)getSeries(series);
			LinearSegment cur = c.getSegment(item);

			if(item > 0 && cur.leftopen) {
				LinearSegment prev = c.getSegment(item-1);
				return prev.f(cur.x0);
			} else {
				return cur.y0;
			}
		} else {
			return getYValue(series, item);
		}
	}

	public double getRate(int series, int item) {
		if(getSeries(series) instanceof Curve) {
			return ((Curve)getSeries(series)).getSegment(item).grad;
		} else {
			return 0.0;
		}
	}
}
