package unikl.disco.tests;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import unikl.disco.misc.GraphUtils;
import unikl.disco.nc.Curve;
import unikl.disco.nc.Flow;
import unikl.disco.nc.NetworkAnalyser;

import edu.uci.ics.jung.algorithms.shortestpath.DijkstraShortestPath;
import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.DirectedSparseGraph;
import edu.uci.ics.jung.graph.impl.SimpleSparseVertex;

public class Node_10Flows_10ACs {
	static final Curve default_service_curve = Curve.createRateLatency( 10, 10 ); // latency, rate
	static final Curve default_max_service_curve = Curve.createRateLatency( 5, 15 ); // latency, rate

	Curve[] arrival_curves = new Curve[10];
	Flow[] flows = new Flow[10];
	
	Curve service_curve = Curve.createRateLatency( 10, 10 ); // latency, rate
	Curve max_service_curve = Curve.createRateLatency( 5, 15 ); // latency, rate
	
	public static void main(String[] args) {
		Node_10Flows_10ACs demo = new Node_10Flows_10ACs();
		demo.run();
	}
	
	public void run() {
		DirectedSparseGraph network = new DirectedSparseGraph();

		SimpleSparseVertex v_f0_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f1_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f2_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f3_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f4_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f5_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f6_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f7_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f8_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f9_src = new SimpleSparseVertex();
		SimpleSparseVertex v0 = new SimpleSparseVertex();
		SimpleSparseVertex v_expl_snk = new SimpleSparseVertex();

		network.addVertex( v_f0_src );
		network.addVertex( v_f1_src );
		network.addVertex( v_f2_src );
		network.addVertex( v_f3_src );
		network.addVertex( v_f4_src );
		network.addVertex( v_f5_src );
		network.addVertex( v_f6_src );
		network.addVertex( v_f7_src );
		network.addVertex( v_f8_src );
		network.addVertex( v_f9_src );
		network.addVertex( v0 );
		network.addVertex( v_expl_snk );
		
		Set<Vertex> servers = new HashSet<Vertex>();
		servers.add( v0 );

		DirectedSparseEdge l_f0_src_v0 = new DirectedSparseEdge( v_f0_src, v0 );
		DirectedSparseEdge l_f1_src_v0 = new DirectedSparseEdge( v_f1_src, v0 );
		DirectedSparseEdge l_f2_src_v0 = new DirectedSparseEdge( v_f2_src, v0 );
		DirectedSparseEdge l_f3_src_v0 = new DirectedSparseEdge( v_f3_src, v0 );
		DirectedSparseEdge l_f4_src_v0 = new DirectedSparseEdge( v_f4_src, v0 );
		DirectedSparseEdge l_f5_src_v0 = new DirectedSparseEdge( v_f5_src, v0 );
		DirectedSparseEdge l_f6_src_v0 = new DirectedSparseEdge( v_f6_src, v0 );
		DirectedSparseEdge l_f7_src_v0 = new DirectedSparseEdge( v_f7_src, v0 );
		DirectedSparseEdge l_f8_src_v0 = new DirectedSparseEdge( v_f8_src, v0 );
		DirectedSparseEdge l_f9_src_v0 = new DirectedSparseEdge( v_f9_src, v0 );
		DirectedSparseEdge l_v0_expl_snk = new DirectedSparseEdge( v0, v_expl_snk );

		network.addEdge( l_f0_src_v0 );
		network.addEdge( l_f1_src_v0 );
		network.addEdge( l_f2_src_v0 );
		network.addEdge( l_f3_src_v0 );
		network.addEdge( l_f4_src_v0 );
		network.addEdge( l_f5_src_v0 );
		network.addEdge( l_f6_src_v0 );
		network.addEdge( l_f7_src_v0 );
		network.addEdge( l_f8_src_v0 );
		network.addEdge( l_f9_src_v0 );
		network.addEdge( l_v0_expl_snk );
		
		NetworkAnalyser na = new NetworkAnalyser( network );
		na.setServiceCurve( servers, service_curve );
		
		DijkstraShortestPath shortest_paths = new DijkstraShortestPath( network );
		Set<Vertex> sources = GraphUtils.getSourceSet( network );
		
		int i = 1;
		for( Vertex source : sources ) {
			List<DirectedEdge> path = shortest_paths.getPath( source, v_expl_snk );
			if ( path.size() > 0 ) {
				arrival_curves[i-1] = Curve.createTokenBucket( i, i*0.1 ); // burst, rate
				flows[i-1] = new Flow( source, v_expl_snk, arrival_curves[i-1].copy(), path );
				na.addFlow( flows[i-1] );
				i++;
			}
		}
		
		na.setUseFifoService( false );
		na.setUseGamma( false );
		na.setUseExtraGamma( false );
		
		Flow flow_of_interest;
		Curve beta;
		double delay;
		Map<Vertex,Double> server_bound_map = new HashMap<Vertex,Double>();
		
//--------------------Flow 0--------------------	
		flow_of_interest = flows[0];
		
		server_bound_map.clear();
		beta = Curve.NULL_CURVE;
		delay = 0.0;
		
		System.out.println( "Flow of interest : " + flow_of_interest.toString() );
		System.out.println();
		
//		TFA
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		server_bound_map.clear();
		delay = na.performTotalFlowAnalysis( flow_of_interest, server_bound_map );
		System.out.println( "delay bound     : " + delay );
		System.out.println( "backlog bound   : " + Collections.max(server_bound_map.values() ) );
		System.out.println( "     per server : " + server_bound_map.values().toString() );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 15.5" );
		System.out.println( "backlog bound   : 110" );
		System.out.println();

//		SFA
		na.setUseFifoService( true );
		
		System.out.println("--- Separated Flow Analysis (FIFO_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 15.6173913043" );
		System.out.println( "backlog bound   : 2.54" );
		System.out.println();

		na.setUseFifoService( false );

		System.out.println("--- Separated Flow Analysis (ARB_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 33.6956521739" );
		System.out.println( "backlog bound   : 4.347826087" );
		System.out.println();
		
//		PMOO
		System.out.println("--- PMOO Analysis (ARB_MUX) ---");
		beta = na.performPMOOAnalysis( flow_of_interest );
		System.out.println( "PMOO service curve: " + beta );
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 33.6956521739" );
		System.out.println( "backlog bound   : 4.347826087" );
		System.out.println();

//--------------------Flow 6--------------------	
		flow_of_interest = flows[6];
		
		server_bound_map.clear();
		beta = Curve.NULL_CURVE;
		delay = 0.0;
		
		System.out.println( "Flow of interest : " + flow_of_interest.toString() );
		System.out.println();
		
//		TFA
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		server_bound_map.clear();
		delay = na.performTotalFlowAnalysis( flow_of_interest, server_bound_map );
		System.out.println( "delay bound     : " + delay );
		System.out.println( "backlog bound   : " + Collections.max(server_bound_map.values() ) );
		System.out.println( "     per server : " + server_bound_map.values().toString() );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 15.5" );
		System.out.println( "backlog bound   : 110" );
		System.out.println();

//		SFA
		na.setUseFifoService( true );
		
		System.out.println("--- Separated Flow Analysis (FIFO_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 16.1461538462" );
		System.out.println( "backlog bound   : 17.36" );
		System.out.println();

		na.setUseFifoService( false );

		System.out.println("--- Separated Flow Analysis (ARB_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 29.8076923077" );
		System.out.println( "backlog bound   : 26.9230769231" );
		System.out.println();
		
//		PMOO
		System.out.println("--- PMOO Analysis (ARB_MUX) ---");
		beta = na.performPMOOAnalysis( flow_of_interest );
		System.out.println( "PMOO service curve: " + beta );
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 29.8076923077" );
		System.out.println( "backlog bound   : 26.9230769231" );
		System.out.println();
	}
}