package unikl.disco.tests;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import unikl.disco.nc.Curve;
import unikl.disco.nc.Flow;
import unikl.disco.nc.NetworkAnalyser;

import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.DirectedSparseGraph;
import edu.uci.ics.jung.graph.impl.SimpleSparseVertex;

public class Node_1Flow {
	static final Curve default_service_curve = Curve.createRateLatency( 10, 10 ); // latency, rate
	static final Curve default_arrival_curve = Curve.createTokenBucket( 25, 5 ); // burst, rate
	static final Curve default_max_service_curve = Curve.createRateLatency( 5, 15 ); // latency, rate

	Curve service_curve = Curve.createRateLatency( 10, 10 ); // latency, rate
	Curve arrival_curve = Curve.createTokenBucket( 25, 5 ); // burst, rate
	Curve max_service_curve = Curve.createRateLatency( 5, 15 ); // latency, rate
	
	public static void main(String[] args) {
		Node_1Flow demo = new Node_1Flow();
		demo.run();
	}
	
	public void run() {
		DirectedSparseGraph network = new DirectedSparseGraph();

		SimpleSparseVertex v_f0_src = new SimpleSparseVertex();
		SimpleSparseVertex v0 = new SimpleSparseVertex();
		SimpleSparseVertex v_expl_snk = new SimpleSparseVertex();

		network.addVertex( v_f0_src );
		network.addVertex( v0 );
		network.addVertex( v_expl_snk );
		
		Set<Vertex> servers = new HashSet<Vertex>();
		servers.add( v0 );
		
		DirectedSparseEdge l_f0_src_v0 = new DirectedSparseEdge( v_f0_src, v0 );
		DirectedSparseEdge l_v0_expl_snk = new DirectedSparseEdge( v0, v_expl_snk );

		network.addEdge( l_f0_src_v0 );
		network.addEdge( l_v0_expl_snk );
		
		NetworkAnalyser na = new NetworkAnalyser( network );
		na.setServiceCurve( servers, service_curve );
		
		LinkedList<DirectedEdge> f0_path = new LinkedList<DirectedEdge>();
		f0_path.add( l_f0_src_v0 );
		f0_path.add( l_v0_expl_snk );
		Flow f0 = new Flow( v_f0_src, v_expl_snk, arrival_curve.copy(), f0_path );
				
		na.addFlow(f0);
		
		na.setUseFifoService( false );
		na.setUseGamma( false );
		na.setUseExtraGamma( false );
		
		Flow flow_of_interest;
		Curve beta;
		double delay;
		Map<Vertex,Double> server_bound_map = new HashMap<Vertex,Double>();
		
//--------------------Flow 0--------------------	
		flow_of_interest = f0;
		
		server_bound_map.clear();
		beta = Curve.NULL_CURVE;
		delay = 0.0;
		
		System.out.println( "Flow of interest : " + flow_of_interest.toString() );
		System.out.println();
		
//		TFA
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		server_bound_map.clear();
		delay = na.performTotalFlowAnalysis( flow_of_interest, server_bound_map );
		System.out.println( "delay bound     : " + delay );
		System.out.println( "backlog bound   : " + Collections.max(server_bound_map.values() ) );
		System.out.println( "     per server : " + server_bound_map.values().toString() );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 12.5" );
		System.out.println( "backlog bound   : 75" );
		System.out.println();

//		SFA
		na.setUseFifoService( true );
		
		System.out.println("--- Separated Flow Analysis (FIFO_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 12.5" );
		System.out.println( "backlog bound   : 75" );
		System.out.println();

		na.setUseFifoService( false );

		System.out.println("--- Separated Flow Analysis (ARB_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 12.5" );
		System.out.println( "backlog bound   : 75" );
		System.out.println();
		
//		PMOO
		System.out.println("--- PMOO Analysis (ARB_MUX) ---");
		beta = na.performPMOOAnalysis( flow_of_interest );
		System.out.println( "PMOO service curve: " + beta );
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 12.5" );
		System.out.println( "backlog bound   : 75" );
		System.out.println();
	}
}