package unikl.disco.tests;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import unikl.disco.nc.Curve;
import unikl.disco.nc.Flow;
import unikl.disco.nc.NetworkAnalyser;

import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.DirectedSparseGraph;
import edu.uci.ics.jung.graph.impl.SimpleSparseVertex;

public class Tandem_1SC_2Flows_1AC_2Paths_v2 {
	static final Curve default_service_curve = Curve.createRateLatency( 10, 10 ); // latency, rate
	static final Curve default_arrival_curve = Curve.createTokenBucket( 25, 5 ); // burst, rate
	static final Curve default_max_service_curve = Curve.createRateLatency( 5, 15 ); // latency, rate

	Curve service_curve = Curve.createRateLatency( 20, 20 ); // latency, rate
	Curve arrival_curve = Curve.createTokenBucket( 25, 5 ); // burst, rate
	Curve max_service_curve = Curve.createRateLatency( 5, 50 ); // latency, rate
	
	public static void main(String[] args) {
		Tandem_1SC_2Flows_1AC_2Paths_v2 demo = new Tandem_1SC_2Flows_1AC_2Paths_v2();
		demo.run();
	}
	
	public void run() {
		DirectedSparseGraph network = new DirectedSparseGraph();

		SimpleSparseVertex v_f0_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f1_src = new SimpleSparseVertex();
		SimpleSparseVertex v0 = new SimpleSparseVertex();
		SimpleSparseVertex v1 = new SimpleSparseVertex();
		SimpleSparseVertex v2 = new SimpleSparseVertex();
		SimpleSparseVertex v3 = new SimpleSparseVertex();
		SimpleSparseVertex v_expl_snk_f0 = new SimpleSparseVertex();
		SimpleSparseVertex v_expl_snk_f1 = new SimpleSparseVertex();

		network.addVertex( v_f0_src );
		network.addVertex( v_f1_src );
		network.addVertex( v0 );
		network.addVertex( v1 );
		network.addVertex( v2 );
		network.addVertex( v3 );
		network.addVertex( v_expl_snk_f0 );
		network.addVertex( v_expl_snk_f1 );
		
		Set<Vertex> servers = new HashSet<Vertex>();
		servers.add( v0 );
		servers.add( v1 );
		servers.add( v2 );
		servers.add( v3 );

		DirectedSparseEdge l_f0_src_v0 = new DirectedSparseEdge( v_f0_src, v0 );
		DirectedSparseEdge l_f1_src_v1 = new DirectedSparseEdge( v_f1_src, v1 );
		DirectedSparseEdge l_v0_v1 = new DirectedSparseEdge( v0, v1 );
		DirectedSparseEdge l_v1_v2 = new DirectedSparseEdge( v1, v2 );
		DirectedSparseEdge l_v2_v3 = new DirectedSparseEdge( v2, v3 );
		DirectedSparseEdge l_v2_expl_snk_f1 = new DirectedSparseEdge( v2, v_expl_snk_f1 );
		DirectedSparseEdge l_v3_expl_snk_f0 = new DirectedSparseEdge( v3, v_expl_snk_f0 );

		network.addEdge( l_f0_src_v0 );
		network.addEdge( l_f1_src_v1 );
		network.addEdge( l_v0_v1 );
		network.addEdge( l_v1_v2 );
		network.addEdge( l_v2_v3 );
		network.addEdge( l_v2_expl_snk_f1 );
		network.addEdge( l_v3_expl_snk_f0 );
		
		NetworkAnalyser na = new NetworkAnalyser( network );
		na.setServiceCurve( servers, service_curve );
		
		LinkedList<DirectedEdge> f0_path = new LinkedList<DirectedEdge>();
		f0_path.add( l_f0_src_v0 );
		f0_path.add( l_v0_v1 );
		f0_path.add( l_v1_v2 );
		f0_path.add( l_v2_v3 );
		f0_path.add( l_v3_expl_snk_f0 );
		Flow f0 = new Flow( v_f0_src, v_expl_snk_f0, arrival_curve.copy(), f0_path );
				
		na.addFlow(f0);
		
		LinkedList<DirectedEdge> f1_path = new LinkedList<DirectedEdge>();
		f1_path.add( l_f1_src_v1 );
		f0_path.add( l_v1_v2 );
		f1_path.add( l_v2_expl_snk_f1 );
		Flow f1 = new Flow( v_f1_src, v_expl_snk_f1, arrival_curve.copy(), f1_path );
				
		na.addFlow(f1);
		
		na.setUseFifoService( false );
		na.setUseGamma( false );
		na.setUseExtraGamma( false );
		
		Flow flow_of_interest;
		Curve beta;
		double delay;
		Map<Vertex,Double> server_bound_map = new HashMap<Vertex,Double>();
		
//--------------------Flow 0--------------------	
		flow_of_interest = f0;
		
		server_bound_map.clear();
		beta = Curve.NULL_CURVE;
		delay = 0.0;
		
		System.out.println( "Flow of interest : " + flow_of_interest.toString() );
		System.out.println();
		
//		TFA
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		server_bound_map.clear();
		delay = na.performTotalFlowAnalysis( flow_of_interest, server_bound_map );
		System.out.println( "delay bound     : " + delay );
		System.out.println( "backlog bound   : " + Collections.max(server_bound_map.values() ) );
		System.out.println( "     per server : " + server_bound_map.values().toString() );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 124.765625" );
		System.out.println( "backlog bound   : 550" );
		System.out.println();

//		SFA
		na.setUseFifoService( true );
		
		System.out.println("--- Separated Flow Analysis (FIFO_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 89.166666" );
		System.out.println( "backlog bound   : 462.5" );
		System.out.println();

		na.setUseFifoService( false );

		System.out.println("--- Separated Flow Analysis (ARB_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 105" );
		System.out.println( "backlog bound   : 541.66666" );
		System.out.println();
		
//		PMOO
		System.out.println("--- PMOO Analysis (ARB_MUX) ---");
		beta = na.performPMOOAnalysis( flow_of_interest );
		System.out.println( "PMOO service curve: " + beta );
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 96.66666" );
		System.out.println( "backlog bound   : 500" );
		System.out.println();

//--------------------Flow 1--------------------	
		flow_of_interest = f1;
		
		server_bound_map.clear();
		beta = Curve.NULL_CURVE;
		delay = 0.0;
		
		System.out.println( "Flow of interest : " + flow_of_interest.toString() );
		System.out.println();
		
//		TFA
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		server_bound_map.clear();
		delay = na.performTotalFlowAnalysis( flow_of_interest, server_bound_map );
		System.out.println( "delay bound     : " + delay );
		System.out.println( "backlog bound   : " + Collections.max(server_bound_map.values() ) );
		System.out.println( "     per server : " + server_bound_map.values().toString() );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 65" );
		System.out.println( "backlog bound   : 550" );
		System.out.println();

//		SFA
		na.setUseFifoService( true );
		
		System.out.println("--- Separated Flow Analysis (FIFO_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 59.166666" );
		System.out.println( "backlog bound   : 312.5" );
		System.out.println();

		na.setUseFifoService( false );

		System.out.println("--- Separated Flow Analysis (ARB_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 78.33333" );
		System.out.println( "backlog bound   : 408.33333" );
		System.out.println();
		
//		PMOO
		System.out.println("--- PMOO Analysis (ARB_MUX) ---");
		beta = na.performPMOOAnalysis( flow_of_interest );
		System.out.println( "PMOO service curve: " + beta );
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 63.33333" );
		System.out.println( "backlog bound   : 333.33333" );
		System.out.println();
	}
}