package unikl.disco.tests;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import unikl.disco.nc.Curve;
import unikl.disco.nc.Flow;
import unikl.disco.nc.NetworkAnalyser;

import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.DirectedSparseGraph;
import edu.uci.ics.jung.graph.impl.SimpleSparseVertex;

public class Tandem_1SC_3Flows_1AC_3Paths {
	Curve service_curve = Curve.createRateLatency( 20, 20 ); // latency, rate
	Curve arrival_curve = Curve.createTokenBucket( 25, 5 ); // burst, rate
	Curve max_service_curve = Curve.createRateLatency( 5, 50 ); // latency, rate
	
	public static void main(String[] args) {
		Tandem_1SC_3Flows_1AC_3Paths demo = new Tandem_1SC_3Flows_1AC_3Paths();
		demo.run();
	}
	
	public void run() {
		DirectedSparseGraph network = new DirectedSparseGraph();

		SimpleSparseVertex v_f0_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f1_src = new SimpleSparseVertex();
		SimpleSparseVertex v_f2_src = new SimpleSparseVertex();
		SimpleSparseVertex v0 = new SimpleSparseVertex();
		SimpleSparseVertex v1 = new SimpleSparseVertex();
		SimpleSparseVertex v2 = new SimpleSparseVertex();
		SimpleSparseVertex v_expl_snk_f0 = new SimpleSparseVertex();
		SimpleSparseVertex v_expl_snk_f1 = new SimpleSparseVertex();
		SimpleSparseVertex v_expl_snk_f2 = new SimpleSparseVertex();

		network.addVertex( v_f0_src );
		network.addVertex( v_f1_src );
		network.addVertex( v_f2_src );
		network.addVertex( v0 );
		network.addVertex( v1 );
		network.addVertex( v2 );
		network.addVertex( v_expl_snk_f0 );
		network.addVertex( v_expl_snk_f1 );
		network.addVertex( v_expl_snk_f2 );
		
		Set<Vertex> servers = new HashSet<Vertex>();
		servers.add( v0 );
		servers.add( v1 );
		servers.add( v2 );

		DirectedSparseEdge l_f0_src_v0 = new DirectedSparseEdge( v_f0_src, v0 );
		DirectedSparseEdge l_f1_src_v2 = new DirectedSparseEdge( v_f1_src, v2 );
		DirectedSparseEdge l_f2_src_v0 = new DirectedSparseEdge( v_f2_src, v0 );
		DirectedSparseEdge l_v0_v1 = new DirectedSparseEdge( v0, v1 );
		DirectedSparseEdge l_v1_v2 = new DirectedSparseEdge( v1, v2 );
		DirectedSparseEdge l_v1_expl_snk_f0 = new DirectedSparseEdge( v1, v_expl_snk_f0 );
		DirectedSparseEdge l_v2_expl_snk_f1 = new DirectedSparseEdge( v2, v_expl_snk_f1 );
		DirectedSparseEdge l_v2_expl_snk_f2 = new DirectedSparseEdge( v2, v_expl_snk_f2 );

		network.addEdge( l_f0_src_v0 );
		network.addEdge( l_f1_src_v2 );
		network.addEdge( l_f2_src_v0 );
		network.addEdge( l_v0_v1 );
		network.addEdge( l_v1_v2 );
		network.addEdge( l_v1_expl_snk_f0 );
		network.addEdge( l_v2_expl_snk_f1 );
		network.addEdge( l_v2_expl_snk_f2 );
		
		NetworkAnalyser na = new NetworkAnalyser( network );
		na.setServiceCurve( servers, service_curve );
		
		LinkedList<DirectedEdge> f0_path = new LinkedList<DirectedEdge>();
		f0_path.add( l_f0_src_v0 );
		f0_path.add( l_v0_v1 );
		f0_path.add( l_v1_expl_snk_f0 );
		Flow f0 = new Flow( v_f0_src, v_expl_snk_f0, arrival_curve.copy(), f0_path );
				
		na.addFlow(f0);

		LinkedList<DirectedEdge> f1_path = new LinkedList<DirectedEdge>();
		f1_path.add( l_f1_src_v2 );
		f1_path.add( l_v2_expl_snk_f1 );
		Flow f1 = new Flow( v_f1_src, v_expl_snk_f1, arrival_curve.copy(), f1_path );
				
		na.addFlow(f1);
		
		LinkedList<DirectedEdge> f2_path = new LinkedList<DirectedEdge>();
		f2_path.add( l_f2_src_v0 );
		f2_path.add( l_v0_v1 );
		f2_path.add( l_v1_v2 );
		f2_path.add( l_v2_expl_snk_f2 );
		Flow f2 = new Flow( v_f2_src, v_expl_snk_f2, arrival_curve.copy(), f2_path );
				
		na.addFlow(f2);
		
		na.setUseFifoService( false );
		na.setUseGamma( false );
		na.setUseExtraGamma( false );
		
		Flow flow_of_interest;
		Curve beta;
		double delay;
		Map<Vertex,Double> server_bound_map = new HashMap<Vertex,Double>();
		
//--------------------Flow 0--------------------	
		flow_of_interest = f0;
		
		server_bound_map.clear();
		beta = Curve.NULL_CURVE;
		delay = 0.0;
		
		System.out.println( "Flow of interest : " + flow_of_interest.toString() );
		System.out.println();
		
//		TFA
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		server_bound_map.clear();
		delay = na.performTotalFlowAnalysis( flow_of_interest, server_bound_map );
		System.out.println( "delay bound     : " + delay );
		System.out.println( "backlog bound   : " + Collections.max(server_bound_map.values() ) );
		System.out.println( "     per server : " + server_bound_map.values().toString() );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 55" );
		System.out.println( "backlog bound   : 450" );
		System.out.println();

//		SFA
		na.setUseFifoService( true );
		
		System.out.println("--- Separated Flow Analysis (FIFO_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 49.166666" );
		System.out.println( "backlog bound   : 262.5" );
		System.out.println();

		na.setUseFifoService( false );

		System.out.println("--- Separated Flow Analysis (ARB_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 65" );
		System.out.println( "backlog bound   : 341.66666" );
		System.out.println();
		
//		PMOO
		System.out.println("--- PMOO Analysis (ARB_MUX) ---");
		beta = na.performPMOOAnalysis( flow_of_interest );
		System.out.println( "PMOO service curve: " + beta );
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 56.66666" );
		System.out.println( "backlog bound   : 300" );
		System.out.println();

//--------------------Flow 1--------------------	
		flow_of_interest = f1;
		
		server_bound_map.clear();
		beta = Curve.NULL_CURVE;
		delay = 0.0;
		
		System.out.println( "Flow of interest : " + flow_of_interest.toString() );
		System.out.println();
		
//		TFA
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		server_bound_map.clear();
		delay = na.performTotalFlowAnalysis( flow_of_interest, server_bound_map );
		System.out.println( "delay bound     : " + delay );
		System.out.println( "backlog bound   : " + Collections.max(server_bound_map.values() ) );
		System.out.println( "     per server : " + server_bound_map.values().toString() );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 34.453125" );
		System.out.println( "backlog bound   : 489.0625" );
		System.out.println();

//		SFA
		na.setUseFifoService( true );
		
		System.out.println("--- Separated Flow Analysis (FIFO_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 34.86979166666" );
		System.out.println( "backlog bound   : 191.015625" );
		System.out.println();

		na.setUseFifoService( false );

		System.out.println("--- Separated Flow Analysis (ARB_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 52.037037037" );
		System.out.println( "backlog bound   : 276.851851851" );
		System.out.println();
		
//		PMOO
		System.out.println("--- PMOO Analysis (ARB_MUX) ---");
		beta = na.performPMOOAnalysis( flow_of_interest );
		System.out.println( "PMOO service curve: " + beta );
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 48.33333" );
		System.out.println( "backlog bound   : 258.33333" );
		System.out.println();

//--------------------Flow 2--------------------	
		flow_of_interest = f2;
		
		server_bound_map.clear();
		beta = Curve.NULL_CURVE;
		delay = 0.0;
		
		System.out.println( "Flow of interest : " + flow_of_interest.toString() );
		System.out.println();
		
//		TFA
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		server_bound_map.clear();
		delay = na.performTotalFlowAnalysis( flow_of_interest, server_bound_map );
		System.out.println( "delay bound     : " + delay );
		System.out.println( "backlog bound   : " + Collections.max(server_bound_map.values() ) );
		System.out.println( "     per server : " + server_bound_map.values().toString() );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 89.453125" );
		System.out.println( "backlog bound   : 489.0625" );
		System.out.println();

//		SFA
		na.setUseFifoService( true );
		
		System.out.println("--- Separated Flow Analysis (FIFO_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 70.4166666" );
		System.out.println( "backlog bound   : 368.75" );
		System.out.println();

		na.setUseFifoService( false );

		System.out.println("--- Separated Flow Analysis (ARB_MUX) ---");
		beta = na.performSeparatedFlowAnalysis( flow_of_interest );
		System.out.println("SFA service curve: " + beta);
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 93.33333" );
		System.out.println( "backlog bound   : 483.33333" );
		System.out.println();
		
//		PMOO
		System.out.println("--- PMOO Analysis (ARB_MUX) ---");
		beta = na.performPMOOAnalysis( flow_of_interest );
		System.out.println( "PMOO service curve: " + beta );
		System.out.println( "delay bound     : " + Curve.getDelayBound( flow_of_interest.ac, beta ) );
		System.out.println( "backlog bound   : " + Curve.getBacklogBound( flow_of_interest.ac, beta ) );
		System.out.println();
		System.out.println( "expected results" );
		System.out.println( "delay bound     : 85" );
		System.out.println( "backlog bound   : 441.66666" );
		System.out.println();
	}
}